@@ -20,9 +20,8 @@ import os
 
 from oslo_log import log as logging
 from oslo_policy import policy
+import pkg_resources
 import stevedore
-from tempest import clients
-from tempest.common import credentials_factory as credentials
 from tempest import config
 
 from patrole_tempest_plugin.rbac_authority import RbacAuthority
@@ -34,6 +33,7 @@ LOG = logging.getLogger(__name__)
 
 class PolicyAuthority(RbacAuthority):
     """A class that uses ``oslo.policy`` for validating RBAC."""
+    os_admin = None
 
     def __init__(self, project_id, user_id, service, extra_target_data=None):
         """Initialization of Policy Authority class.
@@ -123,12 +123,10 @@ class PolicyAuthority(RbacAuthority):
 
         # Cache the list of available services in memory to avoid needlessly
         # doing an API call every time.
-        if not hasattr(cls, 'available_services'):
-            admin_mgr = clients.Manager(
-                credentials.get_configured_admin_credentials())
-            services_client = (admin_mgr.identity_services_v3_client
+        if not hasattr(cls, 'available_services') and cls.os_admin:
+            services_client = (cls.os_admin.identity_services_v3_client
                                if CONF.identity_feature_enabled.api_v3
-                               else admin_mgr.identity_services_client)
+                               else cls.os_admin.identity_services_client)
             services = services_client.list_services()['services']
             cls.available_services = [s['name'] for s in services]
 
@@ -186,10 +184,19 @@ class PolicyAuthority(RbacAuthority):
             }
         )
         LOG.warn(deprecated_msg)
-        default.check = policy.OrCheck(
-            [policy._parser.parse_rule(cs) for cs in
-                [default.check_str,
-                 deprecated_rule.check_str]])
+        oslo_policy_version = pkg_resources.parse_version(
+            pkg_resources.get_distribution("oslo.policy").version)
+        # NOTE(gmann): oslo policy 3.7.0 onwards does not allow to modify
+        # the Rule object check attribute.
+        required_version = pkg_resources.parse_version('3.7.0')
+        if oslo_policy_version >= required_version:
+            return policy.OrCheck([default.check, deprecated_rule.check])
+        else:
+            default.check = policy.OrCheck(
+                [policy._parser.parse_rule(cs) for cs in
+                    [default.check_str,
+                     deprecated_rule.check_str]])
+            return default.check
 
     def get_rules(self):
         rules = policy.Rules()
@@ -229,9 +236,10 @@ class PolicyAuthority(RbacAuthority):
                             # NOTE (sergey.vilgelm):
                             # The `DocumentedRuleDefault` object has no
                             # `deprecated_rule` attribute in Pike
+                            check = rule.check
                             if getattr(rule, 'deprecated_rule', False):
-                                self._handle_deprecated_rule(rule)
-                        rules[rule.name] = rule.check
+                                check = self._handle_deprecated_rule(rule)
+                        rules[rule.name] = check
                     elif str(rule.check) != str(rules[rule.name]):
                         msg = ("The same policy name: %s was found in the "
                                "policies files and in the code for service "
