@@ -20,8 +20,6 @@ import time
 from oslo_log import log as logging
 from oslo_utils import excutils
 
-from tempest import clients
-from tempest.common import credentials_factory as credentials
 from tempest import config
 from tempest.lib import exceptions as lib_exc
 
@@ -128,6 +126,7 @@ class RbacUtilsMixin(object):
     defined by ``CONF.identity.admin_role`` and
     ``CONF.patrole.rbac_test_roles``.
     """
+    credentials = ['primary', 'admin']
 
     def __init__(self, *args, **kwargs):
         super(RbacUtilsMixin, self).__init__(*args, **kwargs)
@@ -143,16 +142,24 @@ class RbacUtilsMixin(object):
     _user_id = None
     _role_map = None
     _role_inferences_mapping = None
+    _orig_roles = []
 
     admin_roles_client = None
 
+    @classmethod
+    def restore_roles(cls):
+        if cls._orig_roles:
+            LOG.info("Restoring original roles %s", cls._orig_roles)
+            roles_already_present = cls._list_and_clear_user_roles_on_project(
+                cls._orig_roles)
+
+            if not roles_already_present:
+                cls._create_user_role_on_project(cls._orig_roles)
+
     @classmethod
     def setup_clients(cls):
-        # Intialize the admin roles_client to perform role switching.
-        admin_mgr = clients.Manager(
-            credentials.get_configured_admin_credentials())
         if CONF.identity_feature_enabled.api_v3:
-            admin_roles_client = admin_mgr.roles_v3_client
+            admin_roles_client = cls.os_admin.roles_v3_client
         else:
             raise lib_exc.InvalidConfiguration(
                 "Patrole role overriding only supports v3 identity API.")
@@ -165,8 +172,15 @@ class RbacUtilsMixin(object):
 
         cls._init_roles()
 
+        # Store the user's original roles and rollback after testing.
+        roles = cls.admin_roles_client.list_user_roles_on_project(
+            cls._project_id, cls._user_id)['roles']
+        cls._orig_roles = [role['id'] for role in roles]
+        cls.addClassResourceCleanup(cls.restore_roles)
+
         # Change default role to admin
         cls._override_role(False)
+
         super(RbacUtilsMixin, cls).setup_clients()
 
     @classmethod
