@@ -11,14 +11,11 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-from __future__ import absolute_import
-
-import functools
-import mock
-from oslo_config import cfg
+from unittest import mock
 
 import fixtures
+import functools
+from oslo_config import cfg
 from tempest.lib import exceptions
 
 from patrole_tempest_plugin import rbac_exceptions
